@@ -6,7 +6,7 @@
  *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: slonik.c,v 1.67.2.17 2008-06-03 18:54:42 wieck Exp $
+ *	$Id: slonik.c,v 1.67.2.19 2009-04-13 17:41:33 wieck Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -2641,6 +2641,16 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	for (i = 0; i < num_nodes; i++)
 	{
 		nodeinfo[i].no_id = (int)strtol(PQgetvalue(res1, i, 0), NULL, 10);
+
+		if (get_adminfo((SlonikStmt *) stmt, nodeinfo[i].no_id) == NULL)
+		{
+			printf("WARNING: no admin conninfo for node %d - node will not be considered\n",
+				nodeinfo[i].no_id);
+			nodeinfo[i].adminfo = NULL;
+			nodeinfo[i].has_slon = false;
+			continue;
+		}
+
 		nodeinfo[i].adminfo = get_active_adminfo((SlonikStmt *) stmt,
 												 nodeinfo[i].no_id);
 		if (nodeinfo[i].adminfo == NULL)
@@ -2681,7 +2691,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	PQclear(res1);
 
 	/*
-	 * For every set we're interested in lookup the direct subscriber nodes.
+	 * For every set we're interested in lookup the subscriber nodes.
 	 */
 	for (i = 0; i < num_sets; i++)
 	{
@@ -2695,11 +2705,9 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 					 "select sub_receiver "
 					 "    from \"_%s\".sl_subscribe "
 					 "    where sub_set = %d "
-					 "    and sub_provider = %d "
 					 "    and sub_active and sub_forward; ",
 					 stmt->hdr.script->clustername,
-					 setinfo[i].set_id,
-					 stmt->no_id);
+					 setinfo[i].set_id);
 
 		res3 = db_exec_select((SlonikStmt *) stmt, adminfo1, &query);
 		if (res3 == NULL)
@@ -2743,6 +2751,8 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	 * Execute the failedNode() procedure, first on the backup node, then on
 	 * all other nodes.
 	 */
+	printf("INFO: calling failedNode(%d,%d) on node %d\n",
+			stmt->no_id, stmt->backup_node, stmt->no_id);
 	slon_mkquery(&query,
 				 "select \"_%s\".failedNode(%d, %d); ",
 				 stmt->hdr.script->clustername,
@@ -2758,6 +2768,11 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 		if (nodeinfo[i].no_id == stmt->backup_node)
 			continue;
 
+		if (nodeinfo[i].adminfo == NULL)
+			continue;
+
+		printf("INFO: calling failedNode(%d,%d) on node %d\n",
+				stmt->no_id, stmt->backup_node, nodeinfo[i].no_id);
 		if (db_exec_command((SlonikStmt *) stmt, nodeinfo[i].adminfo, &query) < 0)
 		{
 			free(configbuf);
@@ -2771,6 +2786,9 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	 */
 	for (i = 0; i < num_nodes; i++)
 	{
+		if (nodeinfo[i].adminfo == NULL)
+			continue;
+
 		if (db_commit_xact((SlonikStmt *) stmt, nodeinfo[i].adminfo) < 0)
 		{
 			free(configbuf);
@@ -2783,6 +2801,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	 * Wait until all slon replication engines that where running have
 	 * restarted.
 	 */
+	printf("INFO: Waiting for slon engines to restart\n");
 	n = 0;
 	while (n < num_nodes)
 	{
@@ -2837,6 +2856,9 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 				 stmt->no_id);
 	for (i = 0; i < num_nodes; i++)
 	{
+		if (nodeinfo[i].adminfo == NULL)
+			continue;
+
 		res1 = db_exec_select((SlonikStmt *) stmt, nodeinfo[i].adminfo, &query);
 		if (res1 != NULL)
 		{
@@ -2858,7 +2880,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	}
 
 	/*
-	 * For every set determine the direct subscriber with the highest applied
+	 * For every set determine the subscriber with the highest applied
 	 * sync, preferring the backup node.
 	 */
 	for (i = 0; i < num_sets; i++)
@@ -2905,6 +2927,9 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 		{
 			int64		ssy_seqno;
 
+			if (setinfo[i].subscribers[j]->adminfo == NULL)
+				continue;
+
 			res1 = db_exec_select((SlonikStmt *) stmt,
 								  setinfo[i].subscribers[j]->adminfo, &query);
 			if (res1 == NULL)
@@ -2982,8 +3007,13 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 			setinfo[i].max_node->num_sets++;
 		}
 
+		printf("INFO: Node with highest sync for set %d is %d\n",
+				setinfo[i].set_id, use_node);
+
 		if (use_node != stmt->backup_node)
 		{
+			printf("INFO: switching node %d to temporarily receive set %d from node %d\n",
+					stmt->backup_node, setinfo[i].set_id, use_node);
 			slon_mkquery(&query,
 						 "select \"_%s\".storeListen(%d,%d,%d); "
 						 "select \"_%s\".subscribeSet_int(%d,%d,%d,true); ",
@@ -3003,7 +3033,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 		rc = -1;
 
 	/*
-	 * Now execute all FAILED_NODE events on the node that had the highest of
+	 * Now execute all FAILED_NODE2 events on the node that had the highest of
 	 * all events alltogether.
 	 */
 	if (max_node_total != NULL)
@@ -3032,6 +3062,9 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	 */
 	for (i = 0; i < num_nodes; i++)
 	{
+		if (nodeinfo[i].adminfo == NULL)
+			continue;
+
 		if (db_commit_xact((SlonikStmt *) stmt, nodeinfo[i].adminfo) < 0)
 			rc = -1;
 	}
